using System;
using System.Net;
using System.Collections;
using CookComputing.XmlRpc;


namespace ExampleClient
{
	/// <summary>
	/// Access ConceptNet using XML RPC
	/// </summary>
	public class ConceptNetClient
	{
		
		debugWindow debug;
		private IConceptNet conceptNetProxy;
		private XmlRpcClientProtocol clientProtocol;
		
		
		public ConceptNetClient(debugWindow _debug)
		{
			debug = _debug;

			conceptNetProxy = (IConceptNet)XmlRpcProxyGen.Create(typeof(IConceptNet));
			clientProtocol = (XmlRpcClientProtocol)conceptNetProxy;
			clientProtocol.Url = "http://localhost:8000/";

		}


		private void HandleException(Exception ex)
		{			
			try
			{
				throw ex;
			}
			catch(Exception e)
			{
				debug.writeLine("Error in ability_commonsenseConceptNet: " + e.Message);
			}
			//			catch(XmlRpcFaultException fex)
			//			{
			//				
			//			}
			//			catch(WebException webEx)
			//			{
			//				
			//			}
			//			catch(XmlRpcServerException xmlRpcEx)
			//			{
			//				
			//			}
			//			catch(Exception defEx)
			//			{
			//				
			//			}
		}
		
		public ArrayList cn_getObjects(string subject, string predicate)
		{
			ArrayList results = new ArrayList();
			//debug.writeLine("in getObjects");
			
			string predicateText = "==" + predicate + "==>";

			string nodeInfo = this.cn_displayNode(subject);
			//debug.writeLine(nodeInfo);
			
			string[] parts = nodeInfo.Split(' ');
			
			//find each predicate
			for(int i=0; i<parts.Length; i++)
			{
				string part = parts[i];
				int startIndex= 0;
				int endIndex = 0;

				//(1) find the start of predicate
				if(part.StartsWith(predicateText))
				{
					startIndex = i;
					//debug.writeLine(part);
					
					//(2) find the end of predicate
					for(int j=i; j<parts.Length; j++)
					{
						string test = parts[j];
						if(test.StartsWith("("))
						{
							endIndex = j;
							break;
						}
					}
				}

				//(3) add the IsA to results
				if(startIndex !=0 && endIndex !=0)
				{
					//reassemble object
					string objectText = "";
					for(int r=startIndex+1; r<endIndex; r++)
					{
						objectText = objectText + " " + parts[r];
					}

					results.Add(objectText);
				}				

				//debug.writeLine(i + "|   " + part);				
			}

			//debug.writeLine("Number of results: " + results.Count);	

			return results;
		}

		public string cn_displayNode(string search)
		{
			return this.conceptNetProxy.display_node(search);
		}	

		public ArrayList cn_getAnalogousConcepts(string search)
		{
			ArrayList results = new ArrayList();

			object[] a1 = this.conceptNetProxy.get_analogous_concepts(search,1);
			for(int i=0; i<a1.Length; i++)
			{
				string s = (string)a1[i];
				results.Add(s);
			}

			return results;
		}

		public ArrayList cn_getContext(string search)
		{
			ArrayList results = new ArrayList();

			object[] a1 = this.conceptNetProxy.get_context(new object[]{search});
			for(int i=0; i<a1.Length; i++)
			{
				object[] a2 = (object[])a1[i];
				string s = (string)a2[0];
				results.Add(s);
			}

			return results;
		}

		public ArrayList cn_projectAffective(ArrayList search)
		{
			ArrayList results = new ArrayList();

			object[] toSend = new object[search.Count];
			for(int i=0; i<search.Count; i++)
			{
				toSend[i] = search[i];
			}

			object[] a1 = this.conceptNetProxy.project_affective(toSend);
			for(int i=0; i<a1.Length; i++)
			{
				object[] a2 = (object[])a1[i];
				string s = (string)a2[0];
				results.Add(s);
			}

			return results;
		}

		public ArrayList cn_projectConsequences(ArrayList search)
		{
			ArrayList results = new ArrayList();

			object[] toSend = new object[search.Count];
			for(int i=0; i<search.Count; i++)
			{
				toSend[i] = search[i];
			}

			object[] a1 = this.conceptNetProxy.project_consequences(toSend);
			for(int i=0; i<a1.Length; i++)
			{
				object[] a2 = (object[])a1[i];
				string s = (string)a2[0];
				results.Add(s);
			}

			return results;
		}

		public ArrayList cn_projectDetails(ArrayList search)
		{
			ArrayList results = new ArrayList();

			object[] toSend = new object[search.Count];
			for(int i=0; i<search.Count; i++)
			{
				toSend[i] = search[i];
			}

			object[] a1 = this.conceptNetProxy.project_details(toSend);
			for(int i=0; i<a1.Length; i++)
			{
				object[] a2 = (object[])a1[i];
				string s = (string)a2[0];
				results.Add(s);
			}

			return results;
		}

		public ArrayList cn_projectSpatial(ArrayList search)
		{
			ArrayList results = new ArrayList();

			object[] toSend = new object[search.Count];
			for(int i=0; i<search.Count; i++)
			{
				toSend[i] = search[i];
			}

			object[] a1 = this.conceptNetProxy.project_spatial(toSend);
			for(int i=0; i<a1.Length; i++)
			{
				object[] a2 = (object[])a1[i];
				string s = (string)a2[0];
				results.Add(s);
			}

			return results;
		}


		interface IConceptNet
		{
			[XmlRpcMethod("display_node")]
			string display_node(string textnode);

			//This is a just a way to call the 4 projection methods:
			//[XmlRpcMethod("get_all_projections")]
			//object[] get_all_projections(object[] search);

			[XmlRpcMethod("get_analogous_concepts")]
			object[] get_analogous_concepts(string search, int p);
			
			[XmlRpcMethod("get_context")]
			object[] get_context(object[] search);

			[XmlRpcMethod("project_affective")]
			object[] project_affective(object[] search);

			[XmlRpcMethod("project_consequences")]
			object[] project_consequences(object[] search);

			[XmlRpcMethod("project_details")]
			object[] project_details(object[] search);

			[XmlRpcMethod("project_spatial")]
			object[] project_spatial(object[] search);
		}
	}
}
