using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Data;
using System.Reflection;
using System.Xml;
using System.IO;

namespace CSwordCompletion_conceptNet
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Label suggestion;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.TextBox to;
		private System.Windows.Forms.TextBox textArea;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuItem4;

		private Image backgroundImage;
		
		
		//the Open Mind Common Sense Network
		ConceptNet conceptnet;
		
		//holds the terms used for prediction
		ArrayList currentContextTerms = new ArrayList();
		

		//holds if there is currently a suggestion available
		bool activeSuggestion = false;
		

		//holds the current word completion suggestion
		String currentNextWord = "";
		
		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//

			//set the background image
			backgroundImage = new Bitmap(Assembly.GetExecutingAssembly().GetManifestResourceStream("CSwordCompletion_conceptNet.back.jpg"));
			this.Refresh();

			//load the contexts into memory
			this.loadContexts();
			
			updateSuggestion("");

			//test finding the contexts for american
			//ArrayList results = this.findRelated("american");
			//debug("American has: " + results.Count + " related terms");
//			String resultText = "";
//			foreach(string term in results)
//			{
//				resultText = resultText + " . " + term;
//			}
//			debug(resultText);


		}
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.textArea = new System.Windows.Forms.TextBox();
			this.suggestion = new System.Windows.Forms.Label();
			this.to = new System.Windows.Forms.TextBox();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.menuItem4 = new System.Windows.Forms.MenuItem();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.Add(this.menuItem1);
			this.mainMenu1.MenuItems.Add(this.menuItem2);
			// 
			// menuItem1
			// 
			this.menuItem1.Text = "Use Word";
			this.menuItem1.Click += new System.EventHandler(this.menuItem1_Click);
			// 
			// textArea
			// 
			this.textArea.Location = new System.Drawing.Point(8, 56);
			this.textArea.Multiline = true;
			this.textArea.Size = new System.Drawing.Size(160, 96);
			this.textArea.Text = "";
			this.textArea.TextChanged += new System.EventHandler(this.textArea_TextChanged);
			// 
			// suggestion
			// 
			this.suggestion.Font = new System.Drawing.Font("Nina", 9.75F, System.Drawing.FontStyle.Bold);
			this.suggestion.Location = new System.Drawing.Point(0, 160);
			this.suggestion.Size = new System.Drawing.Size(176, 20);
			this.suggestion.Text = "Test";
			// 
			// to
			// 
			this.to.Location = new System.Drawing.Point(66, 13);
			this.to.Size = new System.Drawing.Size(102, 25);
			this.to.Text = "";
			// 
			// menuItem2
			// 
			this.menuItem2.MenuItems.Add(this.menuItem3);
			this.menuItem2.MenuItems.Add(this.menuItem4);
			this.menuItem2.Text = "Options";
			// 
			// menuItem3
			// 
			this.menuItem3.Text = "Send";
			// 
			// menuItem4
			// 
			this.menuItem4.Text = "New Message";
			this.menuItem4.Click += new System.EventHandler(this.menuItem4_Click);
			// 
			// Form1
			// 
			this.Controls.Add(this.to);
			this.Controls.Add(this.suggestion);
			this.Controls.Add(this.textArea);
			this.Menu = this.mainMenu1;
			this.Text = "New Message";

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>

		static void Main() 
		{
			Application.Run(new Form1());
		}


		//Draws the image to fit the ClientRectangle area of the form.
		protected override void OnPaint(PaintEventArgs e)
		{
			e.Graphics.DrawImage(backgroundImage, this.ClientRectangle, new Rectangle(0, 0, this.backgroundImage.Width, this.backgroundImage.Height), GraphicsUnit.Pixel);
		}

		private void menuItem1_Click(object sender, System.EventArgs e)
		{
			//replace the last fragement with the suggested word
			String[] terms = textArea.Text.Split(' ');
			String returnText = "";
			for(int i=0; i<terms.Length-1; i++)
			{
				returnText = returnText + (String)terms[i] + " ";
			}
			
			this.textArea.Text = returnText + currentNextWord;

			//move the cursor to the end
			this.textArea.Select(textArea.Text.Length,textArea.Text.Length);

			//clear the suggestion
			updateSuggestion("");
		}

		//creates an instance of the Open Mind Common Sense Network
		private void loadContexts()
		{
			conceptnet = new ConceptNet();
			int nodes = conceptnet.getVocabularySize() * 10;
			MessageBox.Show("System has loaded " + nodes + " ConceptNet nodes","", System.Windows.Forms.MessageBoxButtons.OK, System.Windows.Forms.MessageBoxIcon.Asterisk, System.Windows.Forms.MessageBoxDefaultButton.Button1);
		}


		//Returns the related terms for the given context
		private ArrayList findRelated(String term)
		{			
			ArrayList related = new ArrayList();
			
			//use OMCSNet to find the related terms:
			string[] relatedArray = conceptnet.getContextArray(term);
			foreach(string relatedTerm in relatedArray)
			{
				related.Add(relatedTerm);
			}

			//MessageBox.Show("" + related.Count + " items are related to " + term + ".");
			
			return related;
		}


		//update the possible contexts and predict the current word
		private void textArea_TextChanged(object sender, System.EventArgs e)
		{
			String currentText = textArea.Text;

			//if the text area is empty, clear the currentContextTerms
			if(currentText.Length == 0)
			{
				currentContextTerms.Clear();
			}
			
			//update the current contexts if the last character is a space
			//FIX: account for other punctuation
			if(currentText.EndsWith(" "))
			{
				//get the last word
				String[] terms = currentText.Split(' ');
				String lastWord = terms[terms.Length-2];
				
				//test:
				//debug(lastWord + ".");
				
				//search for terms related to the current context
				ArrayList lastWordContext = this.findRelated(lastWord);

				//test:
				//debug(lastWord + " has " + lastWordContext.Count + " related terms");
				foreach(String newTerm in lastWordContext)
				{
					currentContextTerms.Add(newTerm);
										
				}

							
				//debug("System has " + currentContextTerms.Count + " context terms");

			}
			else //attempt to predict the current word
			{
				//get the start of the last word
				String[] terms = currentText.Split(' ');
				String fragment = terms[terms.Length-1];

				//debug("Fragment is: " + fragment + ".");
				
				//if the fragment is two letters or more, search currentContextTerms for a completion
				if(fragment.Length >= 2)
				{
					String firstLetters = fragment;
					//debug(firstTwoLetters);
					bool suggestionValid = false;
					string suggestion = "";
					
					foreach(String testTerm in currentContextTerms)
					{
						//debug("on: " + testTerm);
						
						//set the current suggestion if they both start with
						//the same first two letters
						if(testTerm.StartsWith(firstLetters))
						{
							suggestionValid = true;
							suggestion = testTerm;
							
						}
						
					}
					//update hte currentSuggestion
					if(suggestionValid)
					{
						updateSuggestion(suggestion);
					}
					else
					{
						updateSuggestion("");
					}
					
				}
				//else clear the suggestion (fragment is not two letters or more)
				else
				{
					updateSuggestion("");
				}

			}

		}

		//updates the suggestion in memory and displaed on the screen
		private void updateSuggestion(String word)
		{
			if(word != "")
			{
				activeSuggestion = true;
				this.currentNextWord = word;
				this.suggestion.Visible = true;
				this.suggestion.Text = word;
				this.menuItem1.Text = "Use Word";
			}
			else
			{
				activeSuggestion = false;
				this.currentNextWord = word;
				this.suggestion.Visible = false;
				this.suggestion.Text = word;
				this.menuItem1.Text = "";
			}

		}

		//writes text to the suggestion lable for debugging
		private void debug(String message)
		{
			//this.to.Text = message;
			//this.suggestion.Text = message;
			//this.textArea.Text = message;
		}

		//clicked new message in the options menu
		private void menuItem4_Click(object sender, System.EventArgs e)
		{
			this.textArea.Text = "";
			updateSuggestion("");
		}

		
	}
}
